# CSDumpFormat.py
#
# Copyright (c) 2014, DFN-CERT Services GmbH
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.


from struct import Struct


# File Format Discription
#
# +------------+--------------+--------------------+-----+--------------------+
# | FileHeader | Stats Record | Datablock Header 1 | ... | Datablock Header n |
# +------------+--------------+--------------------+-----+--------------------+
#
# +--------------------+----------+-----+----------+--------------------+-----+
# | Datablock Header 1 | Record 1 | ... | Record n | Datablock Header n | ... |
# +--------------------+----------+-----+----------+--------------------+-----+

MAGIC = 0xA50C
BLOCK_ID = 31


FILEHEADER = Struct(
    "<"
    "H"  # u16 magic, should be 0xA50C
    "H"  # u16 version, should be 1 (v1.5 format)
    "I"  # u32 flags, bitmask for see NFCAP_FLAG_*
    "I"  # u32 numblocks, count of blocks in the file
    "122s"  # ident, free-form string
    "B"  # u8 environment of the sensor
    "B"  # u8 location of the sensor
    "B"  # u8 data_class of the sensor
    "B"  # u8 data_format
    "B"  # u8 src_mode, sanitization mode
    "B"  # u8 dst_mode, sanitization mode
)

STATSRECORD = Struct(
    "<"
    "Q"  # u64 num flows
    "Q"  # u64 num bytes
    "Q"  # u64 num packets
    "Q"  # u64 num flows tcp
    "Q"  # u64 num flows udp
    "Q"  # u64 num flows icmp
    "Q"  # u64 num flows other
    "Q"  # u64 num flows tcp_impact_info
    "Q"  # u64 num flows tcp_impact_low
    "Q"  # u64 num flows tcp_impact_medium
    "Q"  # u64 num flows tcp_impact_high
    "Q"  # u64 num flows udp_impact_info
    "Q"  # u64 num flows udp_impact_low
    "Q"  # u64 num flows udp_impcat_medium
    "Q"  # u64 num flows udp_impact_high
    "Q"  # u64 num flows icmp_impact_info
    "Q"  # u64 num flows icmp_impcat_low
    "Q"  # u64 num flows icmp_impact_medium
    "Q"  # u64 num flows icmp_impact_high
    "Q"  # u64 num flows other_impact_info
    "Q"  # u64 num flows other_impact_low
    "Q"  # u64 num flows other_impact_medium
    "Q"  # u64 num flows other_impact_high
    "Q"  # u64 num flows tcp_rated
    "Q"  # u64 num flows udp_rated
    "Q"  # u64 num flows icmp_rated
    "Q"  # u64 num fows other_rated
    "Q"  # u64 num bytes tcp
    "Q"  # u64 num bytes udp
    "Q"  # u64 num bytes icmp
    "Q"  # u64 num bytes other
    "Q"  # u64 num bytes rated
    "Q"  # u64 num packets tcp
    "Q"  # u64 num packets udp
    "Q"  # u64 num packets icmp
    "Q"  # u64 num packets other
    "Q"  # u64 num packets rated
    "I"  # u32 first seen, unix time stamp
    "I"  # u32 last seen, unix time stamp
    "H"  # u16 first seen milliseconds
    "H"  # u16 last seen milliseconds
    "I"  # u32 sequence failure
)

BLOCKHEADER = Struct(
    "<"
    "I"  # u32 numrecords, number of records
    "I"  # u32 size of this datablock
    "I"  # u32 organisation (of this block)
    "H"  # u16 id, data block type
    "H"  # u16 pad, padding
)

COMMONRECORD = Struct(
    "<"
    "I"  # u32 flags
    "H"  # u16 recordsize
    "H"  # u16 mark
    "H"  # u16 msec_first
    "H"  # u16 msec_last
    "I"  # u32 first
    "I"  # u32 last
    "BBBB"  # u8 dir, tcp, prot, tos
    "H"  # u16 input
    "H"  # u16 output
    "H"  # u16 srcport
    "H"  # u16 dstport
    "H"  # u16 srcas
    "H"  # u16 dstas
    "B"  # u8 meta
    "B"  # u8 fill0
    "H"  # u16 meta_category
)

RECORD_V4BLOCK = Struct(
    "<"
    "I"  # u32 source ipv4 address
    "I"  # u32 destination ipv4 address
    "I"  # u32 number of packets
    "I"  # u32 number of bytes
)

RECORD_V6BLOCK = Struct(
    "<"
    "Q"  # u64 source ipv6 address ip1a
    "Q"  # u64 source ipb4 address ip1b
    "Q"  # u64 destination ipv6 address ip2a
    "Q"  # u64 destination ipv6 address ip2b
)

RECORD_V6_PACKETS_AND_BYTES = Struct(
    "<"
    "Q"  # packets
    "Q"  # bytes
)

FWZBLOCK = Struct(
    "<"
    "Q"  # u64 analyzer id
    "H"  # u16 sector
    "B"  # u8 add_data_type
    "B"  # u8 fill;
    "I"  # u32 organisationid
)

IDSBLOCK = Struct(
    "<"
    "I"  # u32 signature id
    "B"  # u8 origin
    "B"  # u8 impact
    "B"  # u8 fill1
)

MVBLOCK = Struct(
    "<"
    "B"  # u8 uri[64]
    "B"  # u8 sha1[20]
    "I"  # u32 vuln_id
    "B"  # u8 md5[16]
    "I"  # u32 mv_len
    "H"  # u16 event_count
    "H"  # u16 exploid_id
    "B"  # u8 md_type
    "B"  # u8 fill8[7]
)

ARGOSBLOCK = Struct(
    "<"
    "H"  # u16 arg_version_major
    "H"  # u16 arg_version_minor
    "H"  # u16 arg_alert_type
    "H"  # u16 arg_ethernet_type
    "Q"  # u64 argos_vm_id
    "Q"  # u64 argos_id
    "Q"  # u64 arg_cpu_eip
    "Q"  # u64 arg_cpu_faulty_eip
    "Q"  # u64 arg_cpu_eax
    "Q"  # u64 arg_cpu_ebx
    "Q"  # u64 arg_cpu_ecx
    "Q"  # u64 arg_cpu_edx
    "Q"  # u64 arg_cpu_ebp
    "Q"  # u64 arg_cpu_edi
    "Q"  # u64 arg_cpu_esi
    "Q"  # u64 arg_cpu_esp
    "Q"  # u64 arg_cpu_eflags
)


REVBLOCK = Struct(
    "<"
    "Q"  # u64 rec_packets
    "Q"  # u64 rev_octets
    "B"  # u8 rev_tcpflags
    "B"  # u8 rev_tos
)

PSEUDOBLOCK = Struct(
    "<"
    "B"  # u8 srcip_privacy_mode
    "B"  # u8 dstip_privacy_mode
)

ADBLOCK = Struct(
    "<"
    "B"  # u8 additional data from IDMEF
)
