class DictList(list):
    def __init__(self, *args):
        # let's store our columns in a dict
        self.__cols = {}
        # needed to mark empty cells
        self.__empty = list()
        super(DictList, self).__init__()
        # accept to be called with a list of dicts
        for count, lists in enumerate(args):
            for elem in lists:
                self.append(elem)

    def append(self, item):

        if not isinstance(item, dict):
            raise TypeError('just able to swallow dicts')

        store = list()
        # add to columns if a new column appears
        for e in item:
            if e not in iter(self.__cols.values()):
                self.__cols[len(self.__cols)] = e
        # now there should be a column for every element of the item
        # so we include everything if we just go over the columns
        for e in self.__cols:
            store.append(item.setdefault(self.__cols[e], self.__empty))
        # save it as a list
        return super(DictList, self).append(store)

    def __add__(self, list2):
        if not isinstance(list2, type(self)):
            raise TypeError('just able to concatenate DictLists')

        newlist = DictList()
        for elem in self:
            newlist.append(elem)
        for elem in list2:
            newlist.append(elem)

        return newlist

    def __getitem__(self, index):
        return self.__todict__(super(DictList, self).__getitem__(index))

    def __iter__(self):
        for x in super(DictList, self).__iter__():
            yield self.__todict__(x)

    def __todict__(self, item):
        """
        recreate the dict we put in but stored in a list
        """
        d = dict()
        for i in range(len(item)):
            elem = item[i]
            # don't return empty columns
            if id(elem) != id(self.__empty):
                d[self.__cols[i]] = elem
        return d
