# importer.py
#
# Copyright (c) 2015  DFN-CERT Services GmbH
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.


class Importer(object):
    """Base class importing mixmod data.

    Importer read data from a configured data source. They work as
    iterators returning more items as needed.

    Subclasses can override the get() method to return the next value
    to yield, without having to worry about the iterator protocol.

    """

    def __init__(self, **conf):
        self.conf = conf

    def get(self):
        """Return the next value from the import source."""
        raise RuntimeError("Override the next() method in the subclass")

    def __iter__(self):
        while True:
            data = self.get()
            if data is None:
                break
            yield data


class FileImporter(Importer):
    """Base class for importing mixmod data from a file.

    Reads data objects from a file object and lets a caller iterate
    over the resulting data items.

    """

    def __init__(self, fobj, **conf):
        """Read data from fobj and let the caller iterate of data values."""
        super(FileImporter, self).__init__(**conf)
        self.fobj = fobj
